#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "stm8s.h"
#include "usrlib-uart.h"

#define fMASTER    16000000  // 16MHz
#define TIM2_FREQ  1000      // duty modification cycle
#define TIM3_FREQ  10000     // LED PWM cycle
#define TIM2_ARR   ((u16)(fMASTER/TIM2_FREQ)-1)   // 16MHz/1000-1=1599
#define TIM3_ARR   ((u16)(fMASTER/TIM3_FREQ)-1)   // 16MHz/10000-1=159

void UART_Menu(void);

char TXStr[100];

void main(void)
{
	// clock setting
	CLK_ClockSwitchConfig(CLK_SWITCHMODE_AUTO, CLK_SOURCE_HSI, DISABLE, CLK_CURRENTCLOCKSTATE_DISABLE);
	CLK_HSIPrescalerConfig( CLK_PRESCALER_HSIDIV1 );
	CLK_SYSCLKConfig( CLK_PRESCALER_CPUDIV1 );

	// PWM duty modification cycle timer
	TIM2_DeInit();
	TIM2_TimeBaseInit(TIM2_PRESCALER_1, TIM2_ARR);
	TIM2_ITConfig(TIM2_IT_UPDATE, ENABLE);
	TIM2_Cmd(ENABLE);

	// PWM
	TIM3_DeInit();
	TIM3_TimeBaseInit(TIM3_PRESCALER_1, TIM3_ARR);
	TIM3_Cmd(ENABLE);
	
	UART2_DeInit();
  UART2_Init((u32)115200, UART2_WORDLENGTH_9D, UART2_STOPBITS_1, UART2_PARITY_ODD, UART2_SYNCMODE_CLOCK_DISABLE, UART2_MODE_TXRX_ENABLE);	
  UART2_ITConfig(UART2_IT_RXNE_OR, ENABLE);
	UART2_Cmd(ENABLE);

	enableInterrupts();

	SerialPutString("start\r\n");

  while (1)
	{
	  UART_Menu();
	}
}

// TIM2 interrupt routine (modification of PWM duty)
char mode[10]="updown"; // up | down | updown
float T=1; // LED light up cycle
float TIM3_OC2=0; // PWM threshold level
int slant=1; // modification direction of PWM threshold
void TIM2_Update_isr(void) interrupt 13
{
	float diff; // modification value of PWM threshold
	diff = TIM3_ARR / ( T * TIM2_FREQ );
	TIM3_OC2 += diff*slant; // new PWM threshold
	if(strcmp(mode,"up")==0 && TIM3_OC2>=TIM3_ARR) TIM3_OC2=0; // max?
	else if(strcmp(mode,"down")==0 && TIM3_OC2<=0) TIM3_OC2=TIM3_ARR; // min?
	else // updown mode
	{
		if(TIM3_OC2>=TIM3_ARR) // max?
		{
			TIM3_OC2=TIM3_ARR;
			slant=(-1); // down midification of PWM threshold
		}
		else if(TIM3_OC2<=0) // min?
		{
			TIM3_OC2=0;
			slant=1; // up modification of PWM threshold
		}
	}

	// set new PWM threshold
	TIM3_OC2Init(TIM3_OCMODE_PWM1, TIM3_OUTPUTSTATE_ENABLE, (u16)TIM3_OC2, TIM3_OCPOLARITY_LOW);

	TIM2_ClearFlag(TIM2_FLAG_UPDATE);
}

// UART menu routine
//  1) display menu
//  2) command parse
//  3) change LED light up cycle
void UART_Menu(void)
{
	if(UART_STR_EXIST==1) // there is a command line
	{
		int fieldnumber;
		float a;

		fieldnumber = separate_line(command); // divide a line into words

		// command parse
		//  up 3   (0.01-30)
		//  down 2   (0.01-30)
		//  updown 1   (0.01-30)
		if(strcmp(Field[0],"up")==0 && fieldnumber==2)
		{
			a=atof(Field[1]);
			if(a<0.01 || 30<a) goto NG;
			T=a; // LED light up cycle
			strcpy(mode,Field[0]); // mode = up | down | updown
			slant=1; // modification direction of PWM threshold
			goto RET;
		}
		else if(strcmp(Field[0],"down")==0 && fieldnumber==2)
		{
			a=atof(Field[1]);
			if(a<0.01 || 30<a) goto NG;
			T=a;
			strcpy(mode,Field[0]);
			slant=(-1);
			goto RET;
		}
		else if(strcmp(Field[0],"updown")==0 && fieldnumber==2)
		{
			a=atof(Field[1]);
			if(a<0.01 || 30<a) goto NG;
			T=a/2;
			strcpy(mode,Field[0]);
			slant=1;
			goto RET;
		}
		
	NG:
		SerialPutString("\r\nCOMMAND NG!");
		
	RET:	// menus
		SerialPutString("\r\nupdown 3 ---> light up and darken by 3Hz (0.01-30)\r\n");
		SerialPutString("up 2     ---> light up by 2Hz (0.01-30)\r\n");
		SerialPutString("down 1   ---> darken by 1Hz (0.01-30)\r\n\n");
		UART_STR_EXIST=0;
	}
	return;
}

#ifdef USE_FULL_ASSERT
void assert_failed(u8* file, u32 line)
{ 
  while (1)  	{	}
}
#endif
